"""Custom Exceptions for the NetworkImporterAdapter."""


class AdapterLoadFatalError(Exception):
    """Network Importer Exception."""


class ConfigLoadFatalError(Exception):
    """Network Importer Exception."""
