# -----------------------------------------------
#  WARNING, this file has been auto-generated by the network-importer
#  DO NOT EDIT IT Manually or your changes will be lost
# -----------------------------------------------

AS_Path_Access_Lists: {{AS_Path_Access_Lists|to_yaml_list}}
Authentication_Key_Chains: {{Authentication_Key_Chains|to_yaml_list}}

Community_Lists: {{Community_Lists|to_yaml_list}}

IP6_Access_Lists: {{IP6_Access_Lists|to_yaml_list}}
IP_Access_Lists: {{IP_Access_Lists|to_yaml_list}}
PBR_Policies: {{PBR_Policies|to_yaml_list}}

vendor_family: {{Vendor_Family}}
configuration_format: {{Configuration_Format}}

# -----------------------------------------------
# System Information
#   NTP, TACACS, SYSLOG and DNS 
# -----------------------------------------------
device_type: {{Device_Type}}
domain_name: {{Domain_Name}}

{{NTP|to_yaml_dict}}
{{SNMP|to_yaml_dict}}
{{Syslog|to_yaml_dict}}
{{TACACS|to_yaml_dict}}
{{DNS|to_yaml_dict}}

VRFs:
- TESUTO
- default

# -----------------------------------------------
# Routing Information
#   BGP, routes filters 
# -----------------------------------------------
BGP: 
  {{BGP|to_yaml_dict|lower}}
  
routing_policies: {{Routing_Policies|to_yaml_list}}
route_filter_lists: {{Route_Filter_Lists|to_yaml_list}}
route6_filter_lists: {{Route6_Filter_Lists|to_yaml_list}}

